# Auto generated from metamodel.yaml by pythongen.py version: 0.9.0
# Generation date: 2023-03-08T09:55:01
# Schema: MetaModel
#
# id: MetaModel
# description:
# license:

import dataclasses
import sys
import re
from jsonasobj2 import JsonObj, as_dict
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from linkml_runtime.linkml_model.meta import EnumDefinition, PermissibleValue, PvFormulaOptions

from linkml_runtime.utils.slot import Slot
from linkml_runtime.utils.metamodelcore import empty_list, empty_dict, bnode
from linkml_runtime.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
from linkml_runtime.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from linkml_runtime.utils.formatutils import camelcase, underscore, sfx
from linkml_runtime.utils.enumerations import EnumDefinitionImpl
from rdflib import Namespace, URIRef
from linkml_runtime.utils.curienamespace import CurieNamespace
from linkml_runtime.utils.metamodelcore import Bool, Curie, Decimal, ElementIdentifier, NCName, NodeIdentifier, URI, URIorCURIE, XSDDate, XSDDateTime, XSDTime

metamodel_version = "1.7.0"
version = None

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
METAMODEL = CurieNamespace('MetaModel', 'MetaModel/')
LINKML = CurieNamespace('linkml', 'https://w3id.org/linkml/')
SCHEMA = CurieNamespace('schema', 'http://schema.org/')
SHEX = CurieNamespace('shex', 'http://www.w3.org/ns/shex#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = METAMODEL


# Types
class String(str):
    """ A character string """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = METAMODEL.String


class Integer(int):
    """ An integer """
    type_class_uri = XSD.integer
    type_class_curie = "xsd:integer"
    type_name = "integer"
    type_model_uri = METAMODEL.Integer


class Boolean(Bool):
    """ A binary (true or false) value """
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean"
    type_model_uri = METAMODEL.Boolean


class Float(float):
    """ A real number that conforms to the xsd:float specification """
    type_class_uri = XSD.float
    type_class_curie = "xsd:float"
    type_name = "float"
    type_model_uri = METAMODEL.Float


class Double(float):
    """ A real number that conforms to the xsd:double specification """
    type_class_uri = XSD.double
    type_class_curie = "xsd:double"
    type_name = "double"
    type_model_uri = METAMODEL.Double


class Decimal(Decimal):
    """ A real number with arbitrary precision that conforms to the xsd:decimal specification """
    type_class_uri = XSD.decimal
    type_class_curie = "xsd:decimal"
    type_name = "decimal"
    type_model_uri = METAMODEL.Decimal


class Time(XSDTime):
    """ A time object represents a (local) time of day, independent of any particular day """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "time"
    type_model_uri = METAMODEL.Time


class Date(XSDDate):
    """ a date (year, month and day) in an idealized calendar """
    type_class_uri = XSD.date
    type_class_curie = "xsd:date"
    type_name = "date"
    type_model_uri = METAMODEL.Date


class Datetime(XSDDateTime):
    """ The combination of a date and time """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "datetime"
    type_model_uri = METAMODEL.Datetime


class DateOrDatetime(str):
    """ Either a date or a datetime """
    type_class_uri = LINKML.DateOrDatetime
    type_class_curie = "linkml:DateOrDatetime"
    type_name = "date_or_datetime"
    type_model_uri = METAMODEL.DateOrDatetime


class Uriorcurie(URIorCURIE):
    """ a URI or a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uriorcurie"
    type_model_uri = METAMODEL.Uriorcurie


class Curie(Curie):
    """ a compact URI """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "curie"
    type_model_uri = METAMODEL.Curie


class Uri(URI):
    """ a complete URI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = METAMODEL.Uri


class Ncname(NCName):
    """ Prefix part of CURIE """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "ncname"
    type_model_uri = METAMODEL.Ncname


class Objectidentifier(ElementIdentifier):
    """ A URI or CURIE that represents an object in the model. """
    type_class_uri = SHEX.iri
    type_class_curie = "shex:iri"
    type_name = "objectidentifier"
    type_model_uri = METAMODEL.Objectidentifier


class Nodeidentifier(NodeIdentifier):
    """ A URI, CURIE or BNODE that represents a node in a model. """
    type_class_uri = SHEX.nonLiteral
    type_class_curie = "shex:nonLiteral"
    type_name = "nodeidentifier"
    type_model_uri = METAMODEL.Nodeidentifier


# Class references



@dataclass
class Context(YAMLRoot):
    """
    A Context groups some topics like a Namespace/Package
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = METAMODEL.Context
    class_class_curie: ClassVar[str] = "MetaModel:Context"
    class_name: ClassVar[str] = "Context"
    class_model_uri: ClassVar[URIRef] = METAMODEL.Context

    name: Optional[str] = None
    since: Optional[Union[str, XSDDate]] = None
    updated: Optional[Union[str, XSDDate]] = None
    copyright: Optional[str] = None
    master: Optional[Union[str, URI]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.since is not None and not isinstance(self.since, XSDDate):
            self.since = XSDDate(self.since)

        if self.updated is not None and not isinstance(self.updated, XSDDate):
            self.updated = XSDDate(self.updated)

        if self.copyright is not None and not isinstance(self.copyright, str):
            self.copyright = str(self.copyright)

        if self.master is not None and not isinstance(self.master, URI):
            self.master = URI(self.master)

        super().__post_init__(**kwargs)


@dataclass
class Property(YAMLRoot):
    """
    a Property is a Feature/Attribute of a Topic
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = METAMODEL.Property
    class_class_curie: ClassVar[str] = "MetaModel:Property"
    class_name: ClassVar[str] = "Property"
    class_model_uri: ClassVar[URIRef] = METAMODEL.Property

    name: Optional[str] = None
    label: Optional[str] = None
    type: Optional[str] = None
    index: Optional[float] = None
    sortPos: Optional[float] = None
    primaryKey: Optional[Union[bool, Bool]] = None
    mandatory: Optional[Union[bool, Bool]] = None
    namespace: Optional[str] = None
    size: Optional[float] = None
    uploadable: Optional[Union[bool, Bool]] = None
    defaultValue: Optional[str] = None
    inputType: Optional[str] = None
    allowedValues: Optional[str] = None
    documentation: Optional[str] = None
    values_from: Optional[str] = None
    externalFormatterURI: Optional[str] = None
    showInGrid: Optional[Union[bool, Bool]] = None
    isLink: Optional[Union[bool, Bool]] = None
    nullable: Optional[Union[bool, Bool]] = None
    topic: Optional[Union[dict, "Topic"]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.label is not None and not isinstance(self.label, str):
            self.label = str(self.label)

        if self.type is not None and not isinstance(self.type, str):
            self.type = str(self.type)

        if self.index is not None and not isinstance(self.index, float):
            self.index = float(self.index)

        if self.sortPos is not None and not isinstance(self.sortPos, float):
            self.sortPos = float(self.sortPos)

        if self.primaryKey is not None and not isinstance(self.primaryKey, Bool):
            self.primaryKey = Bool(self.primaryKey)

        if self.mandatory is not None and not isinstance(self.mandatory, Bool):
            self.mandatory = Bool(self.mandatory)

        if self.namespace is not None and not isinstance(self.namespace, str):
            self.namespace = str(self.namespace)

        if self.size is not None and not isinstance(self.size, float):
            self.size = float(self.size)

        if self.uploadable is not None and not isinstance(self.uploadable, Bool):
            self.uploadable = Bool(self.uploadable)

        if self.defaultValue is not None and not isinstance(self.defaultValue, str):
            self.defaultValue = str(self.defaultValue)

        if self.inputType is not None and not isinstance(self.inputType, str):
            self.inputType = str(self.inputType)

        if self.allowedValues is not None and not isinstance(self.allowedValues, str):
            self.allowedValues = str(self.allowedValues)

        if self.documentation is not None and not isinstance(self.documentation, str):
            self.documentation = str(self.documentation)

        if self.values_from is not None and not isinstance(self.values_from, str):
            self.values_from = str(self.values_from)

        if self.externalFormatterURI is not None and not isinstance(self.externalFormatterURI, str):
            self.externalFormatterURI = str(self.externalFormatterURI)

        if self.showInGrid is not None and not isinstance(self.showInGrid, Bool):
            self.showInGrid = Bool(self.showInGrid)

        if self.isLink is not None and not isinstance(self.isLink, Bool):
            self.isLink = Bool(self.isLink)

        if self.nullable is not None and not isinstance(self.nullable, Bool):
            self.nullable = Bool(self.nullable)

        if self.topic is not None and not isinstance(self.topic, Topic):
            self.topic = Topic(**as_dict(self.topic))

        super().__post_init__(**kwargs)


@dataclass
class SMWType(YAMLRoot):
    """
    an SMW_Type is a data type which determines the possible values for that type e.g. a Boolean can hold true/false
    values while a Number can hold 3.1459 or 20. A Page can hold the name of a Wiki page see
    https://semantic-mediawiki.org/wiki/Help:List_of_datatypes
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = METAMODEL.SMWType
    class_class_curie: ClassVar[str] = "MetaModel:SMWType"
    class_name: ClassVar[str] = "SMW_Type"
    class_model_uri: ClassVar[URIRef] = METAMODEL.SMWType

    type: Optional[str] = None
    documentation: Optional[str] = None
    id: Optional[str] = None
    helppage: Optional[Union[str, URI]] = None
    typepage: Optional[str] = None
    javaType: Optional[str] = None
    usedByProperties: Optional[Union[dict, Property]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.type is not None and not isinstance(self.type, str):
            self.type = str(self.type)

        if self.documentation is not None and not isinstance(self.documentation, str):
            self.documentation = str(self.documentation)

        if self.id is not None and not isinstance(self.id, str):
            self.id = str(self.id)

        if self.helppage is not None and not isinstance(self.helppage, URI):
            self.helppage = URI(self.helppage)

        if self.typepage is not None and not isinstance(self.typepage, str):
            self.typepage = str(self.typepage)

        if self.javaType is not None and not isinstance(self.javaType, str):
            self.javaType = str(self.javaType)

        if self.usedByProperties is not None and not isinstance(self.usedByProperties, Property):
            self.usedByProperties = Property(**as_dict(self.usedByProperties))

        super().__post_init__(**kwargs)


@dataclass
class Topic(YAMLRoot):
    """
    A Topic is a Concept/Class/Thing/Entity
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = METAMODEL.Topic
    class_class_curie: ClassVar[str] = "MetaModel:Topic"
    class_name: ClassVar[str] = "Topic"
    class_model_uri: ClassVar[URIRef] = METAMODEL.Topic

    name: Optional[str] = None
    pluralName: Optional[str] = None
    icon: Optional[str] = None
    iconUrl: Optional[str] = None
    documentation: Optional[str] = None
    wikiDocumentation: Optional[str] = None
    defaultstoremode: Optional[str] = None
    listLimit: Optional[float] = None
    cargo: Optional[Union[bool, Bool]] = None
    headerTabs: Optional[Union[bool, Bool]] = None
    context: Optional[Union[dict, Context]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.pluralName is not None and not isinstance(self.pluralName, str):
            self.pluralName = str(self.pluralName)

        if self.icon is not None and not isinstance(self.icon, str):
            self.icon = str(self.icon)

        if self.iconUrl is not None and not isinstance(self.iconUrl, str):
            self.iconUrl = str(self.iconUrl)

        if self.documentation is not None and not isinstance(self.documentation, str):
            self.documentation = str(self.documentation)

        if self.wikiDocumentation is not None and not isinstance(self.wikiDocumentation, str):
            self.wikiDocumentation = str(self.wikiDocumentation)

        if self.defaultstoremode is not None and not isinstance(self.defaultstoremode, str):
            self.defaultstoremode = str(self.defaultstoremode)

        if self.listLimit is not None and not isinstance(self.listLimit, float):
            self.listLimit = float(self.listLimit)

        if self.cargo is not None and not isinstance(self.cargo, Bool):
            self.cargo = Bool(self.cargo)

        if self.headerTabs is not None and not isinstance(self.headerTabs, Bool):
            self.headerTabs = Bool(self.headerTabs)

        if self.context is not None and not isinstance(self.context, Context):
            self.context = Context(**as_dict(self.context))

        super().__post_init__(**kwargs)


@dataclass
class Action(YAMLRoot):
    """
    An action/function/operation to be performed
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = METAMODEL.Action
    class_class_curie: ClassVar[str] = "MetaModel:Action"
    class_name: ClassVar[str] = "Action"
    class_model_uri: ClassVar[URIRef] = METAMODEL.Action

    name: Optional[str] = None
    servicetype: Optional[str] = None
    service: Optional[Union[str, URI]] = None
    inputtype: Optional[str] = None
    input: Optional[str] = None
    actionpage: Optional[str] = None
    output: Optional[str] = None
    engine: Optional[str] = None
    author: Optional[str] = None
    since: Optional[Union[str, XSDDate]] = None
    comment: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.servicetype is not None and not isinstance(self.servicetype, str):
            self.servicetype = str(self.servicetype)

        if self.service is not None and not isinstance(self.service, URI):
            self.service = URI(self.service)

        if self.inputtype is not None and not isinstance(self.inputtype, str):
            self.inputtype = str(self.inputtype)

        if self.input is not None and not isinstance(self.input, str):
            self.input = str(self.input)

        if self.actionpage is not None and not isinstance(self.actionpage, str):
            self.actionpage = str(self.actionpage)

        if self.output is not None and not isinstance(self.output, str):
            self.output = str(self.output)

        if self.engine is not None and not isinstance(self.engine, str):
            self.engine = str(self.engine)

        if self.author is not None and not isinstance(self.author, str):
            self.author = str(self.author)

        if self.since is not None and not isinstance(self.since, XSDDate):
            self.since = XSDDate(self.since)

        if self.comment is not None and not isinstance(self.comment, str):
            self.comment = str(self.comment)

        super().__post_init__(**kwargs)


@dataclass
class TopicLink(YAMLRoot):
    """
    A TopicLink links two Concepts
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = METAMODEL.TopicLink
    class_class_curie: ClassVar[str] = "MetaModel:TopicLink"
    class_name: ClassVar[str] = "TopicLink"
    class_model_uri: ClassVar[URIRef] = METAMODEL.TopicLink

    name: Optional[str] = None
    source: Optional[str] = None
    sourceRole: Optional[str] = None
    sourceMultiple: Optional[Union[bool, Bool]] = None
    sourceDocumentation: Optional[str] = None
    target: Optional[str] = None
    targetRole: Optional[str] = None
    targetMultiple: Optional[Union[bool, Bool]] = None
    targetDocumentation: Optional[str] = None
    masterDetail: Optional[Union[bool, Bool]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.source is not None and not isinstance(self.source, str):
            self.source = str(self.source)

        if self.sourceRole is not None and not isinstance(self.sourceRole, str):
            self.sourceRole = str(self.sourceRole)

        if self.sourceMultiple is not None and not isinstance(self.sourceMultiple, Bool):
            self.sourceMultiple = Bool(self.sourceMultiple)

        if self.sourceDocumentation is not None and not isinstance(self.sourceDocumentation, str):
            self.sourceDocumentation = str(self.sourceDocumentation)

        if self.target is not None and not isinstance(self.target, str):
            self.target = str(self.target)

        if self.targetRole is not None and not isinstance(self.targetRole, str):
            self.targetRole = str(self.targetRole)

        if self.targetMultiple is not None and not isinstance(self.targetMultiple, Bool):
            self.targetMultiple = Bool(self.targetMultiple)

        if self.targetDocumentation is not None and not isinstance(self.targetDocumentation, str):
            self.targetDocumentation = str(self.targetDocumentation)

        if self.masterDetail is not None and not isinstance(self.masterDetail, Bool):
            self.masterDetail = Bool(self.masterDetail)

        super().__post_init__(**kwargs)


# Enumerations


# Slots
class slots:
    pass

slots.name = Slot(uri=METAMODEL.name, name="name", curie=METAMODEL.curie('name'),
                   model_uri=METAMODEL.name, domain=None, range=Optional[str])

slots.since = Slot(uri=METAMODEL.since, name="since", curie=METAMODEL.curie('since'),
                   model_uri=METAMODEL.since, domain=None, range=Optional[Union[str, XSDDate]])

slots.updated = Slot(uri=METAMODEL.updated, name="updated", curie=METAMODEL.curie('updated'),
                   model_uri=METAMODEL.updated, domain=None, range=Optional[Union[str, XSDDate]])

slots.copyright = Slot(uri=METAMODEL.copyright, name="copyright", curie=METAMODEL.curie('copyright'),
                   model_uri=METAMODEL.copyright, domain=None, range=Optional[str])

slots.master = Slot(uri=METAMODEL.master, name="master", curie=METAMODEL.curie('master'),
                   model_uri=METAMODEL.master, domain=None, range=Optional[Union[str, URI]])

slots.label = Slot(uri=METAMODEL.label, name="label", curie=METAMODEL.curie('label'),
                   model_uri=METAMODEL.label, domain=None, range=Optional[str])

slots.type = Slot(uri=METAMODEL.type, name="type", curie=METAMODEL.curie('type'),
                   model_uri=METAMODEL.type, domain=None, range=Optional[str])

slots.index = Slot(uri=METAMODEL.index, name="index", curie=METAMODEL.curie('index'),
                   model_uri=METAMODEL.index, domain=None, range=Optional[float])

slots.sortPos = Slot(uri=METAMODEL.sortPos, name="sortPos", curie=METAMODEL.curie('sortPos'),
                   model_uri=METAMODEL.sortPos, domain=None, range=Optional[float])

slots.primaryKey = Slot(uri=METAMODEL.primaryKey, name="primaryKey", curie=METAMODEL.curie('primaryKey'),
                   model_uri=METAMODEL.primaryKey, domain=None, range=Optional[Union[bool, Bool]])

slots.mandatory = Slot(uri=METAMODEL.mandatory, name="mandatory", curie=METAMODEL.curie('mandatory'),
                   model_uri=METAMODEL.mandatory, domain=None, range=Optional[Union[bool, Bool]])

slots.namespace = Slot(uri=METAMODEL.namespace, name="namespace", curie=METAMODEL.curie('namespace'),
                   model_uri=METAMODEL.namespace, domain=None, range=Optional[str])

slots.size = Slot(uri=METAMODEL.size, name="size", curie=METAMODEL.curie('size'),
                   model_uri=METAMODEL.size, domain=None, range=Optional[float])

slots.uploadable = Slot(uri=METAMODEL.uploadable, name="uploadable", curie=METAMODEL.curie('uploadable'),
                   model_uri=METAMODEL.uploadable, domain=None, range=Optional[Union[bool, Bool]])

slots.defaultValue = Slot(uri=METAMODEL.defaultValue, name="defaultValue", curie=METAMODEL.curie('defaultValue'),
                   model_uri=METAMODEL.defaultValue, domain=None, range=Optional[str])

slots.inputType = Slot(uri=METAMODEL.inputType, name="inputType", curie=METAMODEL.curie('inputType'),
                   model_uri=METAMODEL.inputType, domain=None, range=Optional[str])

slots.allowedValues = Slot(uri=METAMODEL.allowedValues, name="allowedValues", curie=METAMODEL.curie('allowedValues'),
                   model_uri=METAMODEL.allowedValues, domain=None, range=Optional[str])

slots.documentation = Slot(uri=METAMODEL.documentation, name="documentation", curie=METAMODEL.curie('documentation'),
                   model_uri=METAMODEL.documentation, domain=None, range=Optional[str])

slots.values_from = Slot(uri=METAMODEL.values_from, name="values_from", curie=METAMODEL.curie('values_from'),
                   model_uri=METAMODEL.values_from, domain=None, range=Optional[str])

slots.externalFormatterURI = Slot(uri=METAMODEL.externalFormatterURI, name="externalFormatterURI", curie=METAMODEL.curie('externalFormatterURI'),
                   model_uri=METAMODEL.externalFormatterURI, domain=None, range=Optional[str])

slots.showInGrid = Slot(uri=METAMODEL.showInGrid, name="showInGrid", curie=METAMODEL.curie('showInGrid'),
                   model_uri=METAMODEL.showInGrid, domain=None, range=Optional[Union[bool, Bool]])

slots.isLink = Slot(uri=METAMODEL.isLink, name="isLink", curie=METAMODEL.curie('isLink'),
                   model_uri=METAMODEL.isLink, domain=None, range=Optional[Union[bool, Bool]])

slots.nullable = Slot(uri=METAMODEL.nullable, name="nullable", curie=METAMODEL.curie('nullable'),
                   model_uri=METAMODEL.nullable, domain=None, range=Optional[Union[bool, Bool]])

slots.topic = Slot(uri=METAMODEL.topic, name="topic", curie=METAMODEL.curie('topic'),
                   model_uri=METAMODEL.topic, domain=None, range=Optional[Union[dict, Topic]])

slots.id = Slot(uri=METAMODEL.id, name="id", curie=METAMODEL.curie('id'),
                   model_uri=METAMODEL.id, domain=None, range=Optional[str])

slots.helppage = Slot(uri=METAMODEL.helppage, name="helppage", curie=METAMODEL.curie('helppage'),
                   model_uri=METAMODEL.helppage, domain=None, range=Optional[Union[str, URI]])

slots.typepage = Slot(uri=METAMODEL.typepage, name="typepage", curie=METAMODEL.curie('typepage'),
                   model_uri=METAMODEL.typepage, domain=None, range=Optional[str])

slots.javaType = Slot(uri=METAMODEL.javaType, name="javaType", curie=METAMODEL.curie('javaType'),
                   model_uri=METAMODEL.javaType, domain=None, range=Optional[str])

slots.usedByProperties = Slot(uri=METAMODEL.usedByProperties, name="usedByProperties", curie=METAMODEL.curie('usedByProperties'),
                   model_uri=METAMODEL.usedByProperties, domain=None, range=Optional[Union[dict, Property]])

slots.pluralName = Slot(uri=METAMODEL.pluralName, name="pluralName", curie=METAMODEL.curie('pluralName'),
                   model_uri=METAMODEL.pluralName, domain=None, range=Optional[str])

slots.icon = Slot(uri=METAMODEL.icon, name="icon", curie=METAMODEL.curie('icon'),
                   model_uri=METAMODEL.icon, domain=None, range=Optional[str])

slots.iconUrl = Slot(uri=METAMODEL.iconUrl, name="iconUrl", curie=METAMODEL.curie('iconUrl'),
                   model_uri=METAMODEL.iconUrl, domain=None, range=Optional[str])

slots.wikiDocumentation = Slot(uri=METAMODEL.wikiDocumentation, name="wikiDocumentation", curie=METAMODEL.curie('wikiDocumentation'),
                   model_uri=METAMODEL.wikiDocumentation, domain=None, range=Optional[str])

slots.defaultstoremode = Slot(uri=METAMODEL.defaultstoremode, name="defaultstoremode", curie=METAMODEL.curie('defaultstoremode'),
                   model_uri=METAMODEL.defaultstoremode, domain=None, range=Optional[str])

slots.listLimit = Slot(uri=METAMODEL.listLimit, name="listLimit", curie=METAMODEL.curie('listLimit'),
                   model_uri=METAMODEL.listLimit, domain=None, range=Optional[float])

slots.cargo = Slot(uri=METAMODEL.cargo, name="cargo", curie=METAMODEL.curie('cargo'),
                   model_uri=METAMODEL.cargo, domain=None, range=Optional[Union[bool, Bool]])

slots.headerTabs = Slot(uri=METAMODEL.headerTabs, name="headerTabs", curie=METAMODEL.curie('headerTabs'),
                   model_uri=METAMODEL.headerTabs, domain=None, range=Optional[Union[bool, Bool]])

slots.context = Slot(uri=METAMODEL.context, name="context", curie=METAMODEL.curie('context'),
                   model_uri=METAMODEL.context, domain=None, range=Optional[Union[dict, Context]])

slots.servicetype = Slot(uri=METAMODEL.servicetype, name="servicetype", curie=METAMODEL.curie('servicetype'),
                   model_uri=METAMODEL.servicetype, domain=None, range=Optional[str])

slots.service = Slot(uri=METAMODEL.service, name="service", curie=METAMODEL.curie('service'),
                   model_uri=METAMODEL.service, domain=None, range=Optional[Union[str, URI]])

slots.inputtype = Slot(uri=METAMODEL.inputtype, name="inputtype", curie=METAMODEL.curie('inputtype'),
                   model_uri=METAMODEL.inputtype, domain=None, range=Optional[str])

slots.input = Slot(uri=METAMODEL.input, name="input", curie=METAMODEL.curie('input'),
                   model_uri=METAMODEL.input, domain=None, range=Optional[str])

slots.actionpage = Slot(uri=METAMODEL.actionpage, name="actionpage", curie=METAMODEL.curie('actionpage'),
                   model_uri=METAMODEL.actionpage, domain=None, range=Optional[str])

slots.output = Slot(uri=METAMODEL.output, name="output", curie=METAMODEL.curie('output'),
                   model_uri=METAMODEL.output, domain=None, range=Optional[str])

slots.engine = Slot(uri=METAMODEL.engine, name="engine", curie=METAMODEL.curie('engine'),
                   model_uri=METAMODEL.engine, domain=None, range=Optional[str])

slots.author = Slot(uri=METAMODEL.author, name="author", curie=METAMODEL.curie('author'),
                   model_uri=METAMODEL.author, domain=None, range=Optional[str])

slots.comment = Slot(uri=METAMODEL.comment, name="comment", curie=METAMODEL.curie('comment'),
                   model_uri=METAMODEL.comment, domain=None, range=Optional[str])

slots.source = Slot(uri=METAMODEL.source, name="source", curie=METAMODEL.curie('source'),
                   model_uri=METAMODEL.source, domain=None, range=Optional[str])

slots.sourceRole = Slot(uri=METAMODEL.sourceRole, name="sourceRole", curie=METAMODEL.curie('sourceRole'),
                   model_uri=METAMODEL.sourceRole, domain=None, range=Optional[str])

slots.sourceMultiple = Slot(uri=METAMODEL.sourceMultiple, name="sourceMultiple", curie=METAMODEL.curie('sourceMultiple'),
                   model_uri=METAMODEL.sourceMultiple, domain=None, range=Optional[Union[bool, Bool]])

slots.sourceDocumentation = Slot(uri=METAMODEL.sourceDocumentation, name="sourceDocumentation", curie=METAMODEL.curie('sourceDocumentation'),
                   model_uri=METAMODEL.sourceDocumentation, domain=None, range=Optional[str])

slots.target = Slot(uri=METAMODEL.target, name="target", curie=METAMODEL.curie('target'),
                   model_uri=METAMODEL.target, domain=None, range=Optional[str])

slots.targetRole = Slot(uri=METAMODEL.targetRole, name="targetRole", curie=METAMODEL.curie('targetRole'),
                   model_uri=METAMODEL.targetRole, domain=None, range=Optional[str])

slots.targetMultiple = Slot(uri=METAMODEL.targetMultiple, name="targetMultiple", curie=METAMODEL.curie('targetMultiple'),
                   model_uri=METAMODEL.targetMultiple, domain=None, range=Optional[Union[bool, Bool]])

slots.targetDocumentation = Slot(uri=METAMODEL.targetDocumentation, name="targetDocumentation", curie=METAMODEL.curie('targetDocumentation'),
                   model_uri=METAMODEL.targetDocumentation, domain=None, range=Optional[str])

slots.masterDetail = Slot(uri=METAMODEL.masterDetail, name="masterDetail", curie=METAMODEL.curie('masterDetail'),
                   model_uri=METAMODEL.masterDetail, domain=None, range=Optional[Union[bool, Bool]])

slots.context__name = Slot(uri=METAMODEL.name, name="context__name", curie=METAMODEL.curie('name'),
                   model_uri=METAMODEL.context__name, domain=None, range=Optional[str])

slots.context__since = Slot(uri=METAMODEL.since, name="context__since", curie=METAMODEL.curie('since'),
                   model_uri=METAMODEL.context__since, domain=None, range=Optional[Union[str, XSDDate]])

slots.context__updated = Slot(uri=METAMODEL.updated, name="context__updated", curie=METAMODEL.curie('updated'),
                   model_uri=METAMODEL.context__updated, domain=None, range=Optional[Union[str, XSDDate]])

slots.context__copyright = Slot(uri=METAMODEL.copyright, name="context__copyright", curie=METAMODEL.curie('copyright'),
                   model_uri=METAMODEL.context__copyright, domain=None, range=Optional[str])

slots.context__master = Slot(uri=METAMODEL.master, name="context__master", curie=METAMODEL.curie('master'),
                   model_uri=METAMODEL.context__master, domain=None, range=Optional[Union[str, URI]])

slots.property__name = Slot(uri=METAMODEL.name, name="property__name", curie=METAMODEL.curie('name'),
                   model_uri=METAMODEL.property__name, domain=None, range=Optional[str])

slots.property__label = Slot(uri=METAMODEL.label, name="property__label", curie=METAMODEL.curie('label'),
                   model_uri=METAMODEL.property__label, domain=None, range=Optional[str])

slots.property__type = Slot(uri=METAMODEL.type, name="property__type", curie=METAMODEL.curie('type'),
                   model_uri=METAMODEL.property__type, domain=None, range=Optional[str])

slots.property__index = Slot(uri=METAMODEL.index, name="property__index", curie=METAMODEL.curie('index'),
                   model_uri=METAMODEL.property__index, domain=None, range=Optional[float])

slots.property__sortPos = Slot(uri=METAMODEL.sortPos, name="property__sortPos", curie=METAMODEL.curie('sortPos'),
                   model_uri=METAMODEL.property__sortPos, domain=None, range=Optional[float])

slots.property__primaryKey = Slot(uri=METAMODEL.primaryKey, name="property__primaryKey", curie=METAMODEL.curie('primaryKey'),
                   model_uri=METAMODEL.property__primaryKey, domain=None, range=Optional[Union[bool, Bool]])

slots.property__mandatory = Slot(uri=METAMODEL.mandatory, name="property__mandatory", curie=METAMODEL.curie('mandatory'),
                   model_uri=METAMODEL.property__mandatory, domain=None, range=Optional[Union[bool, Bool]])

slots.property__namespace = Slot(uri=METAMODEL.namespace, name="property__namespace", curie=METAMODEL.curie('namespace'),
                   model_uri=METAMODEL.property__namespace, domain=None, range=Optional[str])

slots.property__size = Slot(uri=METAMODEL.size, name="property__size", curie=METAMODEL.curie('size'),
                   model_uri=METAMODEL.property__size, domain=None, range=Optional[float])

slots.property__uploadable = Slot(uri=METAMODEL.uploadable, name="property__uploadable", curie=METAMODEL.curie('uploadable'),
                   model_uri=METAMODEL.property__uploadable, domain=None, range=Optional[Union[bool, Bool]])

slots.property__defaultValue = Slot(uri=METAMODEL.defaultValue, name="property__defaultValue", curie=METAMODEL.curie('defaultValue'),
                   model_uri=METAMODEL.property__defaultValue, domain=None, range=Optional[str])

slots.property__inputType = Slot(uri=METAMODEL.inputType, name="property__inputType", curie=METAMODEL.curie('inputType'),
                   model_uri=METAMODEL.property__inputType, domain=None, range=Optional[str])

slots.property__allowedValues = Slot(uri=METAMODEL.allowedValues, name="property__allowedValues", curie=METAMODEL.curie('allowedValues'),
                   model_uri=METAMODEL.property__allowedValues, domain=None, range=Optional[str])

slots.property__documentation = Slot(uri=METAMODEL.documentation, name="property__documentation", curie=METAMODEL.curie('documentation'),
                   model_uri=METAMODEL.property__documentation, domain=None, range=Optional[str])

slots.property__values_from = Slot(uri=METAMODEL.values_from, name="property__values_from", curie=METAMODEL.curie('values_from'),
                   model_uri=METAMODEL.property__values_from, domain=None, range=Optional[str])

slots.property__externalFormatterURI = Slot(uri=METAMODEL.externalFormatterURI, name="property__externalFormatterURI", curie=METAMODEL.curie('externalFormatterURI'),
                   model_uri=METAMODEL.property__externalFormatterURI, domain=None, range=Optional[str])

slots.property__showInGrid = Slot(uri=METAMODEL.showInGrid, name="property__showInGrid", curie=METAMODEL.curie('showInGrid'),
                   model_uri=METAMODEL.property__showInGrid, domain=None, range=Optional[Union[bool, Bool]])

slots.property__isLink = Slot(uri=METAMODEL.isLink, name="property__isLink", curie=METAMODEL.curie('isLink'),
                   model_uri=METAMODEL.property__isLink, domain=None, range=Optional[Union[bool, Bool]])

slots.property__nullable = Slot(uri=METAMODEL.nullable, name="property__nullable", curie=METAMODEL.curie('nullable'),
                   model_uri=METAMODEL.property__nullable, domain=None, range=Optional[Union[bool, Bool]])

slots.property__topic = Slot(uri=METAMODEL.topic, name="property__topic", curie=METAMODEL.curie('topic'),
                   model_uri=METAMODEL.property__topic, domain=None, range=Optional[Union[dict, Topic]])

slots.sMWType__type = Slot(uri=METAMODEL.type, name="sMWType__type", curie=METAMODEL.curie('type'),
                   model_uri=METAMODEL.sMWType__type, domain=None, range=Optional[str])

slots.sMWType__documentation = Slot(uri=METAMODEL.documentation, name="sMWType__documentation", curie=METAMODEL.curie('documentation'),
                   model_uri=METAMODEL.sMWType__documentation, domain=None, range=Optional[str])

slots.sMWType__id = Slot(uri=METAMODEL.id, name="sMWType__id", curie=METAMODEL.curie('id'),
                   model_uri=METAMODEL.sMWType__id, domain=None, range=Optional[str])

slots.sMWType__helppage = Slot(uri=METAMODEL.helppage, name="sMWType__helppage", curie=METAMODEL.curie('helppage'),
                   model_uri=METAMODEL.sMWType__helppage, domain=None, range=Optional[Union[str, URI]])

slots.sMWType__typepage = Slot(uri=METAMODEL.typepage, name="sMWType__typepage", curie=METAMODEL.curie('typepage'),
                   model_uri=METAMODEL.sMWType__typepage, domain=None, range=Optional[str])

slots.sMWType__javaType = Slot(uri=METAMODEL.javaType, name="sMWType__javaType", curie=METAMODEL.curie('javaType'),
                   model_uri=METAMODEL.sMWType__javaType, domain=None, range=Optional[str])

slots.sMWType__usedByProperties = Slot(uri=METAMODEL.usedByProperties, name="sMWType__usedByProperties", curie=METAMODEL.curie('usedByProperties'),
                   model_uri=METAMODEL.sMWType__usedByProperties, domain=None, range=Optional[Union[dict, Property]])

slots.topic__name = Slot(uri=METAMODEL.name, name="topic__name", curie=METAMODEL.curie('name'),
                   model_uri=METAMODEL.topic__name, domain=None, range=Optional[str])

slots.topic__pluralName = Slot(uri=METAMODEL.pluralName, name="topic__pluralName", curie=METAMODEL.curie('pluralName'),
                   model_uri=METAMODEL.topic__pluralName, domain=None, range=Optional[str])

slots.topic__icon = Slot(uri=METAMODEL.icon, name="topic__icon", curie=METAMODEL.curie('icon'),
                   model_uri=METAMODEL.topic__icon, domain=None, range=Optional[str])

slots.topic__iconUrl = Slot(uri=METAMODEL.iconUrl, name="topic__iconUrl", curie=METAMODEL.curie('iconUrl'),
                   model_uri=METAMODEL.topic__iconUrl, domain=None, range=Optional[str])

slots.topic__documentation = Slot(uri=METAMODEL.documentation, name="topic__documentation", curie=METAMODEL.curie('documentation'),
                   model_uri=METAMODEL.topic__documentation, domain=None, range=Optional[str])

slots.topic__wikiDocumentation = Slot(uri=METAMODEL.wikiDocumentation, name="topic__wikiDocumentation", curie=METAMODEL.curie('wikiDocumentation'),
                   model_uri=METAMODEL.topic__wikiDocumentation, domain=None, range=Optional[str])

slots.topic__defaultstoremode = Slot(uri=METAMODEL.defaultstoremode, name="topic__defaultstoremode", curie=METAMODEL.curie('defaultstoremode'),
                   model_uri=METAMODEL.topic__defaultstoremode, domain=None, range=Optional[str])

slots.topic__listLimit = Slot(uri=METAMODEL.listLimit, name="topic__listLimit", curie=METAMODEL.curie('listLimit'),
                   model_uri=METAMODEL.topic__listLimit, domain=None, range=Optional[float])

slots.topic__cargo = Slot(uri=METAMODEL.cargo, name="topic__cargo", curie=METAMODEL.curie('cargo'),
                   model_uri=METAMODEL.topic__cargo, domain=None, range=Optional[Union[bool, Bool]])

slots.topic__headerTabs = Slot(uri=METAMODEL.headerTabs, name="topic__headerTabs", curie=METAMODEL.curie('headerTabs'),
                   model_uri=METAMODEL.topic__headerTabs, domain=None, range=Optional[Union[bool, Bool]])

slots.topic__context = Slot(uri=METAMODEL.context, name="topic__context", curie=METAMODEL.curie('context'),
                   model_uri=METAMODEL.topic__context, domain=None, range=Optional[Union[dict, Context]])

slots.action__name = Slot(uri=METAMODEL.name, name="action__name", curie=METAMODEL.curie('name'),
                   model_uri=METAMODEL.action__name, domain=None, range=Optional[str])

slots.action__servicetype = Slot(uri=METAMODEL.servicetype, name="action__servicetype", curie=METAMODEL.curie('servicetype'),
                   model_uri=METAMODEL.action__servicetype, domain=None, range=Optional[str])

slots.action__service = Slot(uri=METAMODEL.service, name="action__service", curie=METAMODEL.curie('service'),
                   model_uri=METAMODEL.action__service, domain=None, range=Optional[Union[str, URI]])

slots.action__inputtype = Slot(uri=METAMODEL.inputtype, name="action__inputtype", curie=METAMODEL.curie('inputtype'),
                   model_uri=METAMODEL.action__inputtype, domain=None, range=Optional[str])

slots.action__input = Slot(uri=METAMODEL.input, name="action__input", curie=METAMODEL.curie('input'),
                   model_uri=METAMODEL.action__input, domain=None, range=Optional[str])

slots.action__actionpage = Slot(uri=METAMODEL.actionpage, name="action__actionpage", curie=METAMODEL.curie('actionpage'),
                   model_uri=METAMODEL.action__actionpage, domain=None, range=Optional[str])

slots.action__output = Slot(uri=METAMODEL.output, name="action__output", curie=METAMODEL.curie('output'),
                   model_uri=METAMODEL.action__output, domain=None, range=Optional[str])

slots.action__engine = Slot(uri=METAMODEL.engine, name="action__engine", curie=METAMODEL.curie('engine'),
                   model_uri=METAMODEL.action__engine, domain=None, range=Optional[str])

slots.action__author = Slot(uri=METAMODEL.author, name="action__author", curie=METAMODEL.curie('author'),
                   model_uri=METAMODEL.action__author, domain=None, range=Optional[str])

slots.action__since = Slot(uri=METAMODEL.since, name="action__since", curie=METAMODEL.curie('since'),
                   model_uri=METAMODEL.action__since, domain=None, range=Optional[Union[str, XSDDate]])

slots.action__comment = Slot(uri=METAMODEL.comment, name="action__comment", curie=METAMODEL.curie('comment'),
                   model_uri=METAMODEL.action__comment, domain=None, range=Optional[str])

slots.topicLink__name = Slot(uri=METAMODEL.name, name="topicLink__name", curie=METAMODEL.curie('name'),
                   model_uri=METAMODEL.topicLink__name, domain=None, range=Optional[str])

slots.topicLink__source = Slot(uri=METAMODEL.source, name="topicLink__source", curie=METAMODEL.curie('source'),
                   model_uri=METAMODEL.topicLink__source, domain=None, range=Optional[str])

slots.topicLink__sourceRole = Slot(uri=METAMODEL.sourceRole, name="topicLink__sourceRole", curie=METAMODEL.curie('sourceRole'),
                   model_uri=METAMODEL.topicLink__sourceRole, domain=None, range=Optional[str])

slots.topicLink__sourceMultiple = Slot(uri=METAMODEL.sourceMultiple, name="topicLink__sourceMultiple", curie=METAMODEL.curie('sourceMultiple'),
                   model_uri=METAMODEL.topicLink__sourceMultiple, domain=None, range=Optional[Union[bool, Bool]])

slots.topicLink__sourceDocumentation = Slot(uri=METAMODEL.sourceDocumentation, name="topicLink__sourceDocumentation", curie=METAMODEL.curie('sourceDocumentation'),
                   model_uri=METAMODEL.topicLink__sourceDocumentation, domain=None, range=Optional[str])

slots.topicLink__target = Slot(uri=METAMODEL.target, name="topicLink__target", curie=METAMODEL.curie('target'),
                   model_uri=METAMODEL.topicLink__target, domain=None, range=Optional[str])

slots.topicLink__targetRole = Slot(uri=METAMODEL.targetRole, name="topicLink__targetRole", curie=METAMODEL.curie('targetRole'),
                   model_uri=METAMODEL.topicLink__targetRole, domain=None, range=Optional[str])

slots.topicLink__targetMultiple = Slot(uri=METAMODEL.targetMultiple, name="topicLink__targetMultiple", curie=METAMODEL.curie('targetMultiple'),
                   model_uri=METAMODEL.topicLink__targetMultiple, domain=None, range=Optional[Union[bool, Bool]])

slots.topicLink__targetDocumentation = Slot(uri=METAMODEL.targetDocumentation, name="topicLink__targetDocumentation", curie=METAMODEL.curie('targetDocumentation'),
                   model_uri=METAMODEL.topicLink__targetDocumentation, domain=None, range=Optional[str])

slots.topicLink__masterDetail = Slot(uri=METAMODEL.masterDetail, name="topicLink__masterDetail", curie=METAMODEL.curie('masterDetail'),
                   model_uri=METAMODEL.topicLink__masterDetail, domain=None, range=Optional[Union[bool, Bool]])



# Auto generated from FamilyContext.yaml by pythongen.py version: 0.9.0
# Generation date: 2023-03-08T09:54:40
# Schema: FamilyContext
#
# id: FamilyContext
# description:
# license:

import dataclasses
import sys
import re
from jsonasobj2 import JsonObj, as_dict
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from linkml_runtime.linkml_model.meta import EnumDefinition, PermissibleValue, PvFormulaOptions

from linkml_runtime.utils.slot import Slot
from linkml_runtime.utils.metamodelcore import empty_list, empty_dict, bnode
from linkml_runtime.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
from linkml_runtime.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from linkml_runtime.utils.formatutils import camelcase, underscore, sfx
from linkml_runtime.utils.enumerations import EnumDefinitionImpl
from rdflib import Namespace, URIRef
from linkml_runtime.utils.curienamespace import CurieNamespace
from linkml_runtime.utils.metamodelcore import Bool, Curie, Decimal, ElementIdentifier, NCName, NodeIdentifier, URI, URIorCURIE, XSDDate, XSDDateTime, XSDTime

metamodel_version = "1.7.0"
version = None

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
FAMILYCONTEXT = CurieNamespace('FamilyContext', 'FamilyContext/')
LINKML = CurieNamespace('linkml', 'https://w3id.org/linkml/')
SCHEMA = CurieNamespace('schema', 'http://schema.org/')
SHEX = CurieNamespace('shex', 'http://www.w3.org/ns/shex#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = FAMILYCONTEXT


# Types
class String(str):
    """ A character string """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = FAMILYCONTEXT.String


class Integer(int):
    """ An integer """
    type_class_uri = XSD.integer
    type_class_curie = "xsd:integer"
    type_name = "integer"
    type_model_uri = FAMILYCONTEXT.Integer


class Boolean(Bool):
    """ A binary (true or false) value """
    type_class_uri = XSD.boolean
    type_class_curie = "xsd:boolean"
    type_name = "boolean"
    type_model_uri = FAMILYCONTEXT.Boolean


class Float(float):
    """ A real number that conforms to the xsd:float specification """
    type_class_uri = XSD.float
    type_class_curie = "xsd:float"
    type_name = "float"
    type_model_uri = FAMILYCONTEXT.Float


class Double(float):
    """ A real number that conforms to the xsd:double specification """
    type_class_uri = XSD.double
    type_class_curie = "xsd:double"
    type_name = "double"
    type_model_uri = FAMILYCONTEXT.Double


class Decimal(Decimal):
    """ A real number with arbitrary precision that conforms to the xsd:decimal specification """
    type_class_uri = XSD.decimal
    type_class_curie = "xsd:decimal"
    type_name = "decimal"
    type_model_uri = FAMILYCONTEXT.Decimal


class Time(XSDTime):
    """ A time object represents a (local) time of day, independent of any particular day """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "time"
    type_model_uri = FAMILYCONTEXT.Time


class Date(XSDDate):
    """ a date (year, month and day) in an idealized calendar """
    type_class_uri = XSD.date
    type_class_curie = "xsd:date"
    type_name = "date"
    type_model_uri = FAMILYCONTEXT.Date


class Datetime(XSDDateTime):
    """ The combination of a date and time """
    type_class_uri = XSD.dateTime
    type_class_curie = "xsd:dateTime"
    type_name = "datetime"
    type_model_uri = FAMILYCONTEXT.Datetime


class DateOrDatetime(str):
    """ Either a date or a datetime """
    type_class_uri = LINKML.DateOrDatetime
    type_class_curie = "linkml:DateOrDatetime"
    type_name = "date_or_datetime"
    type_model_uri = FAMILYCONTEXT.DateOrDatetime


class Uriorcurie(URIorCURIE):
    """ a URI or a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uriorcurie"
    type_model_uri = FAMILYCONTEXT.Uriorcurie


class Curie(Curie):
    """ a compact URI """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "curie"
    type_model_uri = FAMILYCONTEXT.Curie


class Uri(URI):
    """ a complete URI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = FAMILYCONTEXT.Uri


class Ncname(NCName):
    """ Prefix part of CURIE """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "ncname"
    type_model_uri = FAMILYCONTEXT.Ncname


class Objectidentifier(ElementIdentifier):
    """ A URI or CURIE that represents an object in the model. """
    type_class_uri = SHEX.iri
    type_class_curie = "shex:iri"
    type_name = "objectidentifier"
    type_model_uri = FAMILYCONTEXT.Objectidentifier


class Nodeidentifier(NodeIdentifier):
    """ A URI, CURIE or BNODE that represents a node in a model. """
    type_class_uri = SHEX.nonLiteral
    type_class_curie = "shex:nonLiteral"
    type_name = "nodeidentifier"
    type_model_uri = FAMILYCONTEXT.Nodeidentifier


# Class references



@dataclass
class Family(YAMLRoot):
    """
    In most societies, the family is the principal institution for the socialization of children.
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = FAMILYCONTEXT.Family
    class_class_curie: ClassVar[str] = "FamilyContext:Family"
    class_name: ClassVar[str] = "Family"
    class_model_uri: ClassVar[URIRef] = FAMILYCONTEXT.Family

    name: Optional[str] = None
    weddingDate: Optional[Union[str, XSDDate]] = None
    weddingPlace: Optional[str] = None
    yearMarried: Optional[float] = None
    monthMarried: Optional[float] = None
    divorced: Optional[Union[str, XSDDate]] = None
    husbandOf: Optional[str] = None
    wifeOf: Optional[str] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.weddingDate is not None and not isinstance(self.weddingDate, XSDDate):
            self.weddingDate = XSDDate(self.weddingDate)

        if self.weddingPlace is not None and not isinstance(self.weddingPlace, str):
            self.weddingPlace = str(self.weddingPlace)

        if self.yearMarried is not None and not isinstance(self.yearMarried, float):
            self.yearMarried = float(self.yearMarried)

        if self.monthMarried is not None and not isinstance(self.monthMarried, float):
            self.monthMarried = float(self.monthMarried)

        if self.divorced is not None and not isinstance(self.divorced, XSDDate):
            self.divorced = XSDDate(self.divorced)

        if self.husbandOf is not None and not isinstance(self.husbandOf, str):
            self.husbandOf = str(self.husbandOf)

        if self.wifeOf is not None and not isinstance(self.wifeOf, str):
            self.wifeOf = str(self.wifeOf)

        super().__post_init__(**kwargs)


@dataclass
class Person(YAMLRoot):
    """
    A Person is a human being
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = FAMILYCONTEXT.Person
    class_class_curie: ClassVar[str] = "FamilyContext:Person"
    class_name: ClassVar[str] = "Person"
    class_model_uri: ClassVar[URIRef] = FAMILYCONTEXT.Person

    qid: Optional[str] = None
    royal92id: Optional[str] = None
    name: Optional[str] = None
    nobleTitle: Optional[str] = None
    picture: Optional[Union[str, URI]] = None
    sex: Optional[str] = None
    born: Optional[Union[str, XSDDate]] = None
    yearBorn: Optional[float] = None
    monthBorn: Optional[float] = None
    birthPlace: Optional[str] = None
    died: Optional[Union[str, XSDDate]] = None
    diedAt: Optional[str] = None
    yearDied: Optional[float] = None
    monthDied: Optional[float] = None
    noInLine: Optional[float] = None
    wikiPedia: Optional[Union[str, URI]] = None
    childOf: Optional[Union[dict, Family]] = None
    parentOf: Optional[Union[dict, Family]] = None
    spouse: Optional[Union[dict, "Person"]] = None

    def __post_init__(self, *_: List[str], **kwargs: Dict[str, Any]):
        if self.qid is not None and not isinstance(self.qid, str):
            self.qid = str(self.qid)

        if self.royal92id is not None and not isinstance(self.royal92id, str):
            self.royal92id = str(self.royal92id)

        if self.name is not None and not isinstance(self.name, str):
            self.name = str(self.name)

        if self.nobleTitle is not None and not isinstance(self.nobleTitle, str):
            self.nobleTitle = str(self.nobleTitle)

        if self.picture is not None and not isinstance(self.picture, URI):
            self.picture = URI(self.picture)

        if self.sex is not None and not isinstance(self.sex, str):
            self.sex = str(self.sex)

        if self.born is not None and not isinstance(self.born, XSDDate):
            self.born = XSDDate(self.born)

        if self.yearBorn is not None and not isinstance(self.yearBorn, float):
            self.yearBorn = float(self.yearBorn)

        if self.monthBorn is not None and not isinstance(self.monthBorn, float):
            self.monthBorn = float(self.monthBorn)

        if self.birthPlace is not None and not isinstance(self.birthPlace, str):
            self.birthPlace = str(self.birthPlace)

        if self.died is not None and not isinstance(self.died, XSDDate):
            self.died = XSDDate(self.died)

        if self.diedAt is not None and not isinstance(self.diedAt, str):
            self.diedAt = str(self.diedAt)

        if self.yearDied is not None and not isinstance(self.yearDied, float):
            self.yearDied = float(self.yearDied)

        if self.monthDied is not None and not isinstance(self.monthDied, float):
            self.monthDied = float(self.monthDied)

        if self.noInLine is not None and not isinstance(self.noInLine, float):
            self.noInLine = float(self.noInLine)

        if self.wikiPedia is not None and not isinstance(self.wikiPedia, URI):
            self.wikiPedia = URI(self.wikiPedia)

        if self.childOf is not None and not isinstance(self.childOf, Family):
            self.childOf = Family(**as_dict(self.childOf))

        if self.parentOf is not None and not isinstance(self.parentOf, Family):
            self.parentOf = Family(**as_dict(self.parentOf))

        if self.spouse is not None and not isinstance(self.spouse, Person):
            self.spouse = Person(**as_dict(self.spouse))

        super().__post_init__(**kwargs)


# Enumerations


# Slots
class slots:
    pass

slots.name = Slot(uri=FAMILYCONTEXT.name, name="name", curie=FAMILYCONTEXT.curie('name'),
                   model_uri=FAMILYCONTEXT.name, domain=None, range=Optional[str])

slots.weddingDate = Slot(uri=FAMILYCONTEXT.weddingDate, name="weddingDate", curie=FAMILYCONTEXT.curie('weddingDate'),
                   model_uri=FAMILYCONTEXT.weddingDate, domain=None, range=Optional[Union[str, XSDDate]])

slots.weddingPlace = Slot(uri=FAMILYCONTEXT.weddingPlace, name="weddingPlace", curie=FAMILYCONTEXT.curie('weddingPlace'),
                   model_uri=FAMILYCONTEXT.weddingPlace, domain=None, range=Optional[str])

slots.yearMarried = Slot(uri=FAMILYCONTEXT.yearMarried, name="yearMarried", curie=FAMILYCONTEXT.curie('yearMarried'),
                   model_uri=FAMILYCONTEXT.yearMarried, domain=None, range=Optional[float])

slots.monthMarried = Slot(uri=FAMILYCONTEXT.monthMarried, name="monthMarried", curie=FAMILYCONTEXT.curie('monthMarried'),
                   model_uri=FAMILYCONTEXT.monthMarried, domain=None, range=Optional[float])

slots.divorced = Slot(uri=FAMILYCONTEXT.divorced, name="divorced", curie=FAMILYCONTEXT.curie('divorced'),
                   model_uri=FAMILYCONTEXT.divorced, domain=None, range=Optional[Union[str, XSDDate]])

slots.husbandOf = Slot(uri=FAMILYCONTEXT.husbandOf, name="husbandOf", curie=FAMILYCONTEXT.curie('husbandOf'),
                   model_uri=FAMILYCONTEXT.husbandOf, domain=None, range=Optional[str])

slots.wifeOf = Slot(uri=FAMILYCONTEXT.wifeOf, name="wifeOf", curie=FAMILYCONTEXT.curie('wifeOf'),
                   model_uri=FAMILYCONTEXT.wifeOf, domain=None, range=Optional[str])

slots.qid = Slot(uri=FAMILYCONTEXT.qid, name="qid", curie=FAMILYCONTEXT.curie('qid'),
                   model_uri=FAMILYCONTEXT.qid, domain=None, range=Optional[str])

slots.royal92id = Slot(uri=FAMILYCONTEXT.royal92id, name="royal92id", curie=FAMILYCONTEXT.curie('royal92id'),
                   model_uri=FAMILYCONTEXT.royal92id, domain=None, range=Optional[str])

slots.nobleTitle = Slot(uri=FAMILYCONTEXT.nobleTitle, name="nobleTitle", curie=FAMILYCONTEXT.curie('nobleTitle'),
                   model_uri=FAMILYCONTEXT.nobleTitle, domain=None, range=Optional[str])

slots.picture = Slot(uri=FAMILYCONTEXT.picture, name="picture", curie=FAMILYCONTEXT.curie('picture'),
                   model_uri=FAMILYCONTEXT.picture, domain=None, range=Optional[Union[str, URI]])

slots.sex = Slot(uri=FAMILYCONTEXT.sex, name="sex", curie=FAMILYCONTEXT.curie('sex'),
                   model_uri=FAMILYCONTEXT.sex, domain=None, range=Optional[str])

slots.born = Slot(uri=FAMILYCONTEXT.born, name="born", curie=FAMILYCONTEXT.curie('born'),
                   model_uri=FAMILYCONTEXT.born, domain=None, range=Optional[Union[str, XSDDate]])

slots.yearBorn = Slot(uri=FAMILYCONTEXT.yearBorn, name="yearBorn", curie=FAMILYCONTEXT.curie('yearBorn'),
                   model_uri=FAMILYCONTEXT.yearBorn, domain=None, range=Optional[float])

slots.monthBorn = Slot(uri=FAMILYCONTEXT.monthBorn, name="monthBorn", curie=FAMILYCONTEXT.curie('monthBorn'),
                   model_uri=FAMILYCONTEXT.monthBorn, domain=None, range=Optional[float])

slots.birthPlace = Slot(uri=FAMILYCONTEXT.birthPlace, name="birthPlace", curie=FAMILYCONTEXT.curie('birthPlace'),
                   model_uri=FAMILYCONTEXT.birthPlace, domain=None, range=Optional[str])

slots.died = Slot(uri=FAMILYCONTEXT.died, name="died", curie=FAMILYCONTEXT.curie('died'),
                   model_uri=FAMILYCONTEXT.died, domain=None, range=Optional[Union[str, XSDDate]])

slots.diedAt = Slot(uri=FAMILYCONTEXT.diedAt, name="diedAt", curie=FAMILYCONTEXT.curie('diedAt'),
                   model_uri=FAMILYCONTEXT.diedAt, domain=None, range=Optional[str])

slots.yearDied = Slot(uri=FAMILYCONTEXT.yearDied, name="yearDied", curie=FAMILYCONTEXT.curie('yearDied'),
                   model_uri=FAMILYCONTEXT.yearDied, domain=None, range=Optional[float])

slots.monthDied = Slot(uri=FAMILYCONTEXT.monthDied, name="monthDied", curie=FAMILYCONTEXT.curie('monthDied'),
                   model_uri=FAMILYCONTEXT.monthDied, domain=None, range=Optional[float])

slots.noInLine = Slot(uri=FAMILYCONTEXT.noInLine, name="noInLine", curie=FAMILYCONTEXT.curie('noInLine'),
                   model_uri=FAMILYCONTEXT.noInLine, domain=None, range=Optional[float])

slots.wikiPedia = Slot(uri=FAMILYCONTEXT.wikiPedia, name="wikiPedia", curie=FAMILYCONTEXT.curie('wikiPedia'),
                   model_uri=FAMILYCONTEXT.wikiPedia, domain=None, range=Optional[Union[str, URI]])

slots.childOf = Slot(uri=FAMILYCONTEXT.childOf, name="childOf", curie=FAMILYCONTEXT.curie('childOf'),
                   model_uri=FAMILYCONTEXT.childOf, domain=None, range=Optional[Union[dict, Family]])

slots.parentOf = Slot(uri=FAMILYCONTEXT.parentOf, name="parentOf", curie=FAMILYCONTEXT.curie('parentOf'),
                   model_uri=FAMILYCONTEXT.parentOf, domain=None, range=Optional[Union[dict, Family]])

slots.spouse = Slot(uri=FAMILYCONTEXT.spouse, name="spouse", curie=FAMILYCONTEXT.curie('spouse'),
                   model_uri=FAMILYCONTEXT.spouse, domain=None, range=Optional[Union[dict, Person]])

slots.family__name = Slot(uri=FAMILYCONTEXT.name, name="family__name", curie=FAMILYCONTEXT.curie('name'),
                   model_uri=FAMILYCONTEXT.family__name, domain=None, range=Optional[str])

slots.family__weddingDate = Slot(uri=FAMILYCONTEXT.weddingDate, name="family__weddingDate", curie=FAMILYCONTEXT.curie('weddingDate'),
                   model_uri=FAMILYCONTEXT.family__weddingDate, domain=None, range=Optional[Union[str, XSDDate]])

slots.family__weddingPlace = Slot(uri=FAMILYCONTEXT.weddingPlace, name="family__weddingPlace", curie=FAMILYCONTEXT.curie('weddingPlace'),
                   model_uri=FAMILYCONTEXT.family__weddingPlace, domain=None, range=Optional[str])

slots.family__yearMarried = Slot(uri=FAMILYCONTEXT.yearMarried, name="family__yearMarried", curie=FAMILYCONTEXT.curie('yearMarried'),
                   model_uri=FAMILYCONTEXT.family__yearMarried, domain=None, range=Optional[float])

slots.family__monthMarried = Slot(uri=FAMILYCONTEXT.monthMarried, name="family__monthMarried", curie=FAMILYCONTEXT.curie('monthMarried'),
                   model_uri=FAMILYCONTEXT.family__monthMarried, domain=None, range=Optional[float])

slots.family__divorced = Slot(uri=FAMILYCONTEXT.divorced, name="family__divorced", curie=FAMILYCONTEXT.curie('divorced'),
                   model_uri=FAMILYCONTEXT.family__divorced, domain=None, range=Optional[Union[str, XSDDate]])

slots.family__husbandOf = Slot(uri=FAMILYCONTEXT.husbandOf, name="family__husbandOf", curie=FAMILYCONTEXT.curie('husbandOf'),
                   model_uri=FAMILYCONTEXT.family__husbandOf, domain=None, range=Optional[str])

slots.family__wifeOf = Slot(uri=FAMILYCONTEXT.wifeOf, name="family__wifeOf", curie=FAMILYCONTEXT.curie('wifeOf'),
                   model_uri=FAMILYCONTEXT.family__wifeOf, domain=None, range=Optional[str])

slots.person__qid = Slot(uri=FAMILYCONTEXT.qid, name="person__qid", curie=FAMILYCONTEXT.curie('qid'),
                   model_uri=FAMILYCONTEXT.person__qid, domain=None, range=Optional[str])

slots.person__royal92id = Slot(uri=FAMILYCONTEXT.royal92id, name="person__royal92id", curie=FAMILYCONTEXT.curie('royal92id'),
                   model_uri=FAMILYCONTEXT.person__royal92id, domain=None, range=Optional[str])

slots.person__name = Slot(uri=FAMILYCONTEXT.name, name="person__name", curie=FAMILYCONTEXT.curie('name'),
                   model_uri=FAMILYCONTEXT.person__name, domain=None, range=Optional[str])

slots.person__nobleTitle = Slot(uri=FAMILYCONTEXT.nobleTitle, name="person__nobleTitle", curie=FAMILYCONTEXT.curie('nobleTitle'),
                   model_uri=FAMILYCONTEXT.person__nobleTitle, domain=None, range=Optional[str])

slots.person__picture = Slot(uri=FAMILYCONTEXT.picture, name="person__picture", curie=FAMILYCONTEXT.curie('picture'),
                   model_uri=FAMILYCONTEXT.person__picture, domain=None, range=Optional[Union[str, URI]])

slots.person__sex = Slot(uri=FAMILYCONTEXT.sex, name="person__sex", curie=FAMILYCONTEXT.curie('sex'),
                   model_uri=FAMILYCONTEXT.person__sex, domain=None, range=Optional[str])

slots.person__born = Slot(uri=FAMILYCONTEXT.born, name="person__born", curie=FAMILYCONTEXT.curie('born'),
                   model_uri=FAMILYCONTEXT.person__born, domain=None, range=Optional[Union[str, XSDDate]])

slots.person__yearBorn = Slot(uri=FAMILYCONTEXT.yearBorn, name="person__yearBorn", curie=FAMILYCONTEXT.curie('yearBorn'),
                   model_uri=FAMILYCONTEXT.person__yearBorn, domain=None, range=Optional[float])

slots.person__monthBorn = Slot(uri=FAMILYCONTEXT.monthBorn, name="person__monthBorn", curie=FAMILYCONTEXT.curie('monthBorn'),
                   model_uri=FAMILYCONTEXT.person__monthBorn, domain=None, range=Optional[float])

slots.person__birthPlace = Slot(uri=FAMILYCONTEXT.birthPlace, name="person__birthPlace", curie=FAMILYCONTEXT.curie('birthPlace'),
                   model_uri=FAMILYCONTEXT.person__birthPlace, domain=None, range=Optional[str])

slots.person__died = Slot(uri=FAMILYCONTEXT.died, name="person__died", curie=FAMILYCONTEXT.curie('died'),
                   model_uri=FAMILYCONTEXT.person__died, domain=None, range=Optional[Union[str, XSDDate]])

slots.person__diedAt = Slot(uri=FAMILYCONTEXT.diedAt, name="person__diedAt", curie=FAMILYCONTEXT.curie('diedAt'),
                   model_uri=FAMILYCONTEXT.person__diedAt, domain=None, range=Optional[str])

slots.person__yearDied = Slot(uri=FAMILYCONTEXT.yearDied, name="person__yearDied", curie=FAMILYCONTEXT.curie('yearDied'),
                   model_uri=FAMILYCONTEXT.person__yearDied, domain=None, range=Optional[float])

slots.person__monthDied = Slot(uri=FAMILYCONTEXT.monthDied, name="person__monthDied", curie=FAMILYCONTEXT.curie('monthDied'),
                   model_uri=FAMILYCONTEXT.person__monthDied, domain=None, range=Optional[float])

slots.person__noInLine = Slot(uri=FAMILYCONTEXT.noInLine, name="person__noInLine", curie=FAMILYCONTEXT.curie('noInLine'),
                   model_uri=FAMILYCONTEXT.person__noInLine, domain=None, range=Optional[float])

slots.person__wikiPedia = Slot(uri=FAMILYCONTEXT.wikiPedia, name="person__wikiPedia", curie=FAMILYCONTEXT.curie('wikiPedia'),
                   model_uri=FAMILYCONTEXT.person__wikiPedia, domain=None, range=Optional[Union[str, URI]])

slots.person__childOf = Slot(uri=FAMILYCONTEXT.childOf, name="person__childOf", curie=FAMILYCONTEXT.curie('childOf'),
                   model_uri=FAMILYCONTEXT.person__childOf, domain=None, range=Optional[Union[dict, Family]])

slots.person__parentOf = Slot(uri=FAMILYCONTEXT.parentOf, name="person__parentOf", curie=FAMILYCONTEXT.curie('parentOf'),
                   model_uri=FAMILYCONTEXT.person__parentOf, domain=None, range=Optional[Union[dict, Family]])

slots.person__spouse = Slot(uri=FAMILYCONTEXT.spouse, name="person__spouse", curie=FAMILYCONTEXT.curie('spouse'),
                   model_uri=FAMILYCONTEXT.person__spouse, domain=None, range=Optional[Union[dict, Person]])
